// ライセンス: GPL2

// View履歴で使う構造体
//

#ifndef _VIEW_HISTORYITEM_H
#define _VIEW_HISTORYITEM_H

#include <string>

namespace HISTORY
{
    struct ViewHistoryItem
    {
        std::string url;
        std::string title;
    };
}

#endif
