// ライセンス: GPL2

// 情報表示部

#ifndef _DRAWAREAINFO_H
#define _DRAWAREAINFO_H

#include "drawareabase.h"

namespace ARTICLE
{
    class DrawAreaInfo : public ARTICLE::DrawAreaBase
    {
      public:
        explicit DrawAreaInfo( const std::string& url );
    };
}


#endif

