// ライセンス: GPL2

// メイン表示部

#ifndef _DRAWAREAMAIN_H
#define _DRAWAREAMAIN_H

#include "drawareabase.h"

namespace ARTICLE
{
    class DrawAreaMain : public ARTICLE::DrawAreaBase
    {
      public:
        explicit DrawAreaMain( const std::string& url );
    };
}


#endif

