// ビューのアドレスで使用するサイン

#ifndef _SIGN_H
#define _SIGN_H

#define ARTICLE_SIGN "_ARTICLE_"
#define NEXT_SIGN    "_NEXT_"
#define LOG_SIGN     "_LOG_"
#define SIDEBAR_SIGN "_SIDEBAR_"
#define BOARD_SIGN   "_BOARD_"
#define RES_SIGN     "_RES_"
#define NAME_SIGN    "_NAME_"
#define ID_SIGN      "_ID_"
#define BOOKMK_SIGN  "_BM_"
#define POST_SIGN    "_POST_"
#define HIGHREFRES_SIGN \
                     "_HIGH_REFRES_"
#define URL_SIGN     "_URL_"
#define REFER_SIGN   "_REF_"
#define KEYWORD_SIGN "_KW_"
#define POSTLOG_SIGN "_POST_"
#define ORMODE_SIGN  "_OR_"
#define CENTER_SIGN  "_CENTER_"
#define TIME_SIGN    "_TIME_"
#define TITLE_SIGN   "_TITLE_"

#endif
