// CoreやAdminクラスで使うコマンド構造体

#ifndef COMMAND_ARGS_H
#define COMMAND_ARGS_H

struct COMMAND_ARGS
{
    std::string command;
    std::string url;
    std::string arg1;
    std::string arg2;
    std::string arg3;
    std::string arg4;
    std::string arg5;
    std::string arg6;
    std::string arg7;
    std::string arg8;
};

#endif
